<?php

use App\Models\User;
use Illuminate\Auth\Events\PasswordReset;
use Illuminate\Foundation\Auth\EmailVerificationRequest;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Password;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\Str;

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/

// Route used for email verification
Route::get('/email/verify/{id}/{hash}', function (Request $request, $id) {
    $user = User::findOrFail($id);
    if (User::findOrFail($id)->hasVerifiedEmail())
        return view('already_validated');

    User::findOrFail($id)->markEmailAsVerified();
    return view('validated');
})->middleware(['signed'])->name('verification.verify');

Route::get('/recover/{token}', function (string $token) {
    return view('recover', ['token' => $token]);
})->middleware('guest')->name('password.reset');

Route::post('/reset-password', function (Request $request) {
    $request->validate([
        'token' => 'required',
        'email' => 'required|email',
        'password' => 'required|string|min:6|confirmed',
    ]);

    $status = Password::reset(
        $request->only('email', 'password', 'password_confirmation', 'token'),
        function (User $user, string $password) {
            $user->forceFill([
                'password' => Hash::make($password)
            ]);

            $user->save();

            event(new PasswordReset($user));
        }
    );

    if ($status == Password::INVALID_USER || $status == Password::INVALID_TOKEN) {
        return response()->view('recovered', [
            "message" => __($status)
        ], 401);
    } else if ($status == Password::PASSWORD_RESET) {
        return response()->view('recovered', [
            "message" => __($status)
        ], 200);
    }
})->name('password.update');
