<?php

namespace Tests\Feature;

use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Illuminate\Support\Facades\URL;
use Tests\TestCase;

use App\Models\User;
use Illuminate\Support\Facades\Hash;

class LoginTest extends TestCase
{
    use RefreshDatabase;

    public function test_empty() {
        
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $response = $this->postJson("/api/auth/login", []);
        $response->assertStatus(422);
    }

    public function test_invalid_email() {
        
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $response = $this->postJson("/api/auth/login", [
            "email" => "testfdfgf",
            "password" => "123"
        ]);
        $response->assertStatus(422);
    }

    public function test_wrong_password() {
        
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $response = $this->postJson("/api/auth/login", [
            "email" => "test@localhost",
            "password" => "123"
        ]);
        $response->assertStatus(401);
    }

    public function test_good() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $user->markEmailAsVerified();

        $response = $this->postJson("/api/auth/login", [
            "email" => "test@localhost",
            "password" => "test",
        ]);
        
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "token",
            "user" => [
                "id",
                "username",
                "email",
                "created_at",
                "updated_at"
            ]
        ]);
    }

    public function test_email_not_validated()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $response = $this->postJson("/api/auth/login", [
            "email" => "test@localhost",
            "password" => "test",
        ]);
        $response->assertStatus(401);
    }

    public function test_email_validated()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $response = $this->get(URL::signedRoute('verification.verify', ['id' => $user->id, 'hash' => 'aaa']));
        $response->assertStatus(200);

        $response = $this->postJson("/api/auth/login", [
            "email" => "test@localhost",
            "password" => "test",
        ]);
        $response->assertStatus(200);
    }
}
